import React, { useState, useEffect, useContext } from 'react'
import { withClient, Client } from 'cozy-client'
import {
  ChallengeType,
  UserChallenge,
  ChallengeState,
} from 'services/dataChallengeContracts'
import ChallengeManager from 'services/challengeDataManagerService'
import StyledChallengeSpinner from 'components/CommonKit/Spinner/StyledChallengeSpinner'
import ChallengeListItem from 'components/ContentComponents/ChallengeList/ChallengeListItem'
import ChallengeModal from 'components/ContentComponents/ChallengeModal/ChallengeModal'
import { AppContext } from 'components/Contexts/AppContextProvider'
import { ScreenType } from 'enum/screen.enum'

interface ChallengesListProps {
  client: Client
}

const ChallengesList: React.FC<ChallengesListProps> = ({
  client,
}: ChallengesListProps) => {
  const { fluidTypes, refreshCurrentChallenge, screenType } = useContext(
    AppContext
  )
  const challengeManager = new ChallengeManager(client)
  const [
    ongoingChallenge,
    setOngoingChallenge,
  ] = useState<UserChallenge | null>(null)
  const [challengesType, setChallengesType] = useState<ChallengeType[] | null>(
    null
  )
  const [userChallenges, setUserChallenges] = useState<UserChallenge[] | null>(
    null
  )
  const [openChallengeModal, setOpenChallengeModal] = useState(false)
  const [scroll, setScroll] = useState(0)
  const [paddingBottom, setPaddingBottom] = useState(0)

  const handleCloseClick = () => {
    setOngoingChallenge(null)
    setOpenChallengeModal(false)
  }

  const setRightChallengeInTheMiddle = (
    challengesType: ChallengeType[],
    userChallenges: UserChallenge[]
  ) => {
    const header = screenType === ScreenType.MOBILE ? 96 : 174
    const bottomNav = screenType === ScreenType.MOBILE ? 60 : 0
    const challengeCard = screenType === ScreenType.MOBILE ? 100 : 120
    const paddingBottom =
      (window.innerHeight - header - bottomNav) / 2 -
      userChallenges.length * challengeCard
    const canFitInHalfScreen =
      window.innerHeight / 2 - header >
      (challengesType.length - 1) * challengeCard
    if (userChallenges[0].state === 0) {
      setPaddingBottom(paddingBottom + bottomNav)
      if (canFitInHalfScreen) {
        setScroll(0)
      } else {
        setScroll(
          (challengeCard / 2) *
            challengesType.length *
            (1 + challengeCard / window.innerHeight)
        )
      }
    } else {
      setPaddingBottom(paddingBottom - challengeCard + bottomNav)
      if (canFitInHalfScreen) {
        setScroll(0)
      } else {
        setScroll(
          (challengeCard / 2) *
            (challengesType.length - 1) *
            (1 + challengeCard / window.innerHeight)
        )
      }
    }
  }

  useEffect(() => {
    let subscribed = true
    async function loadAvailableChallenges() {
      const dataAllCT = await challengeManager.getAvailableChallenges(
        fluidTypes
      )
      const dataAllUC = await challengeManager.getAllUserChallenges()

      if (subscribed && dataAllCT && dataAllUC) {
        const ongoingChallengeTmp = dataAllUC.filter(
          challenge => challenge.state === ChallengeState.ONGOING
        )[0]
        setRightChallengeInTheMiddle(dataAllCT, dataAllUC)
        setChallengesType(dataAllCT)
        setUserChallenges(dataAllUC)
        setOngoingChallenge(ongoingChallengeTmp)

        if (
          await challengeManager.isChallengeOver(
            ongoingChallengeTmp,
            fluidTypes
          )
        ) {
          setOpenChallengeModal(true)
          await challengeManager.endChallenge(ongoingChallengeTmp, fluidTypes)
          await refreshCurrentChallenge()
        }
      }
    }
    loadAvailableChallenges()
    return () => {
      subscribed = false
    }
  }, [])

  useEffect(() => {
    let subscribed = true
    async function loadNewChallengesType() {
      const dataAllCT = await challengeManager.getAvailableChallenges(
        fluidTypes
      )
      const dataAllUC = await challengeManager.getAllUserChallenges()
      if (subscribed && dataAllCT && dataAllUC) {
        setChallengesType(dataAllCT)
        setUserChallenges(dataAllUC)
        setRightChallengeInTheMiddle(dataAllCT, dataAllUC)
      }
    }
    loadNewChallengesType()
    return () => {
      subscribed = false
    }
  }, [ongoingChallenge])

  window.scrollTo({
    top: scroll,
    behavior: 'smooth',
  })

  return (
    <>
      {!challengesType ? (
        <div className="content-view-loading">
          <StyledChallengeSpinner size="5em" />
        </div>
      ) : (
        <div className="loc-root">
          <div className="loc-content">
            <div className="timeline-line"></div>
            <div
              className="list-of-challenge-cards"
              style={{ paddingBottom: paddingBottom }}
            >
              {challengesType.map((challenge, index) =>
                index === challengesType.length - 1 && !ongoingChallenge ? (
                  <ChallengeListItem
                    key={index}
                    challenge={challenge}
                    challengeState="available"
                  />
                ) : ongoingChallenge &&
                  challengesType.length - 1 === index ? null : (
                  <ChallengeListItem
                    key={index}
                    challenge={challenge}
                    challengeState="locked"
                  />
                )
              )}
              {userChallenges &&
                userChallenges.map((challenge, index) =>
                  challenge && challenge.state === 0 ? (
                    <ChallengeListItem
                      key={index}
                      challenge={challenge.challengeType}
                      userChallenge={challenge}
                      challengeState="ongoing"
                    />
                  ) : (
                    <ChallengeListItem
                      key={index}
                      challenge={challenge.challengeType}
                      userChallenge={challenge}
                      challengeState="finished"
                      badgeStatus={challenge.badge}
                    />
                  )
                )}
            </div>
          </div>
        </div>
      )}
      <ChallengeModal
        opened={openChallengeModal}
        challenge={ongoingChallenge}
        handleCloseClick={handleCloseClick}
        badgeStatus={ongoingChallenge && ongoingChallenge.badge}
      />
    </>
  )
}

export default withClient(ChallengesList)
