import React, { useState, useEffect, useRef } from 'react'
import { AccountService } from 'services/accountService'

import { JobService, JobState } from 'services/jobsService'
import { FluidType } from 'enum/fluid.enum'

import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { getPicto, getAddPicto, getFuildType, getParamPicto } from 'utils/utils'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import chevronUp from 'assets/icons/ico/chevron-up.svg'

import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'

import failurePicto from 'components/ContentComponents/KonnectorViewer/picto-failure.png'
import IFluidConfig from 'services/IFluidConfig'

import KonnectorNotFound from 'components/ContentComponents/Konnector/KonnectorNotFound'
import KonnectorForm from 'components/ContentComponents/Konnector/KonnectorForm'
import KonnectorResult from 'components/ContentComponents/Konnector/KonnectorResult'

import { Konnector, Trigger } from 'doctypes'

interface KonnectorViewerCardProps {
  fluidConfig: IFluidConfig
  konnector: Konnector
  client: Client
  isParam: boolean
  t: Function
}

const KonnectorViewerCard: React.FC<KonnectorViewerCardProps> = ({
  fluidConfig,
  konnector,
  client,
  isParam,
  t,
}: KonnectorViewerCardProps) => {
  const [account, setAccount] = useState<Account | null>(null)

  const [setActive, setActiveState] = useState('')
  const [setHeight, setHeightState] = useState('0px')

  const [error, setError] = useState<string>('')
  const [jobState, setJobState] = useState<string>('')

  const content: React.MutableRefObject<null> = useRef<null>(null)
  const type: string = fluidConfig.konnectorConfig.type
  const fluid: FluidType = getFuildType(type)
  const iconType = getPicto(fluid)

  const iconAddType = isParam ? getParamPicto(fluid) : getAddPicto(fluid)

  const toggleAccordion = () => {
    setActiveState(setActive === '' ? 'active' : '')
    setHeightState(
      setActive === 'active' &&
        content &&
        content.current &&
        content.current.scrollHeight
        ? '0px'
        : `${content.current.scrollHeight}px`
    )
  }

  const getKonnectorStateMarkup = () => {
    return jobState === JobState.Errored ? (
      <img className="state-icon" src={failurePicto}></img>
    ) : (
      ''
    )
  }

  const handleConnexion = (_account: Account) => {
    setAccount(_account)
  }

  const handleJobState = (_jobState: JobState) => {
    setJobState(_jobState)
  }

  useEffect(() => {
    let subscribed = true
    async function getData() {
      if (konnector) {
        const _account = await AccountService.getAccountByType(
          client,
          konnector.slug
        )
        if (subscribed && _account) {
          setAccount(_account)
        }
      }
    }
    getData()
    return () => {
      subscribed = false
    }
  }, [])

  return (
    <>
      <div className={`accordion ${setActive}`}>
        <div>
          <div
            className={`accordion-header ${setActive}`}
            onClick={toggleAccordion}
          >
            <div className="accordion-icon">
              {!account ? (
                <StyledIcon className="icon" icon={iconAddType} size={49} />
              ) : (
                <StyledIcon className="icon" icon={iconType} size={49} />
              )}
            </div>
            <div className="state-picto">{getKonnectorStateMarkup()}</div>
            <div className="accordion-info">
              <div className="accordion-title text-16-normal">
                {!account
                  ? t('KONNECTORCONFIG.LABEL_CONNECTTO_' + FluidType[fluid])
                  : t('FLUID.' + FluidType[fluid] + '.LABEL')}
              </div>
            </div>
            <StyledIconButton icon={setActive ? chevronUp : chevronDown} />
          </div>

          <div
            ref={content}
            style={{ maxHeight: `${setHeight}` }}
            className={`accordion-content ${setActive}`}
          >
            {!konnector ? (
              <KonnectorNotFound
                konnectorSlug={fluidConfig.konnectorConfig.slug}
              />
            ) : !account ? (
              <KonnectorForm
                fluidConfig={fluidConfig}
                konnector={konnector}
                handleConnexion={handleConnexion}
              />
            ) : (
              <KonnectorResult
                account={account}
                konnector={konnector}
                handleJobState={handleJobState}
              />
            )}
          </div>
        </div>
      </div>
    </>
  )
}
export default translate()(withClient(KonnectorViewerCard))
